#include "AfcFltConfig.h"
#include <QSettings>

static const QString ui_set_is_manual {QStringLiteral("IsManual")};
static const QString ui_set_enabled   {QStringLiteral("Enabled")};
static const QString ui_set_f0        {QStringLiteral("F0")};
static const QString ui_set_fd0       {QStringLiteral("FD0")};
static const QString ui_set_channels  {QStringLiteral("Chs")};
static const QString ui_set_k         {QStringLiteral("K")};

AfcFltConfig::AfcFltConfig() {
    for (int ch_num = 0; ch_num < ch_cnt; ++ch_num) {
        m_k[ch_num] = defaultK();
    }
}

AfcFltConfig::AfcFltConfig(const AfcFltConfig &cfg) :
    m_manual{cfg.m_manual},
    m_enabled{cfg.m_enabled},
    m_f0{cfg.m_f0},
    m_fd0{cfg.m_fd0} {

    for (int ch_num = 0; ch_num < ch_cnt; ++ch_num) {
        m_k[ch_num] = cfg.m_k[ch_num];
    }
}



void AfcFltConfig::save(QSettings &set) const {
    set.setValue(ui_set_is_manual, m_manual);
    set.setValue(ui_set_enabled, m_enabled);
    set.setValue(ui_set_f0, m_f0);
    set.setValue(ui_set_fd0, m_fd0);
    set.beginWriteArray(ui_set_channels, ch_cnt);
    for (int ch_num = 0; ch_num < ch_cnt; ++ch_num) {
        set.setArrayIndex(ch_num);
        set.setValue(ui_set_k, m_k[ch_num]);
    }
    set.endArray();
}

void AfcFltConfig::load(QSettings &set) {
    m_manual = set.value(ui_set_is_manual).toBool();
    m_enabled = set.value(ui_set_enabled).toBool();
    m_f0 = set.value(ui_set_f0, 10000).toDouble();
    m_fd0 = set.value(ui_set_fd0, 192000).toDouble();
    int cnt = set.beginReadArray(ui_set_channels);
    for (int ch_num = 0; (ch_num < ch_cnt) && (ch_num < cnt); ch_num++) {
        set.setArrayIndex(ch_num);
        m_k[ch_num] = set.value(ui_set_k, defaultK()).toDouble();
    }
    set.endArray();
}
