#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QList>
#include <QComboBox>
#include <QDoubleSpinBox>
#include <QLabel>
#include "lqmeas/devs/LTR/modules/LTR35/LTR35.h"
#include "AfcFltConfig.h"

class QGridLayout;
class QCheckBox;
class LTRModuleSelectionUI;



namespace Ui {
class MainWindow;
}

class LQxtTableLoggerEngine;



class MainWindow : public QMainWindow {
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();


public Q_SLOTS:
    void reset();
private Q_SLOTS:
    void closeCurModule();
    void onBeforeModuleChanged();
    void onModuleChanged();

    void setManualUi();
    void manualUpdateUiDev();
    void manualUpdateChUi(int ch);
    void manualUpdateUi();
    void manualConnectSignals(bool en);

    void saveSignalSettings();
    void loadSignalSettings();

    void allChEnChanged(int state);
    void allChPhaValChanged(double pha);
    void allChFreqValChanged(double freq);
    void allChAmpTypeChanged(int type);
    void allChAmpValChanged(double amp);
    void allChOffsValChanged(double offs);
    void allChDutyValChanged(double val);
    void allChSigTypeChanged(int ind);
    void allChRangeChanged(int ind);


    void rangeChanged(int idx);



    void onSetSigButtonClicked();

    void setSignals(bool resetMode = false);    
    void disableChannels();
    void showConfigDialog();
    void uiEnStateUpdate();
protected:
    void closeEvent ( QCloseEvent * event ) override;
private:
    static const unsigned dac_ch_max_cnt = 8;
    static const unsigned stop_gen_tout = 30000;
    void applyAfcParams();

    QSharedPointer<LQMeas::LTR35> curDev();

    enum DAC_MODES {
        DAC_MODE_CYCLE = 0,
        DAC_MODE_ATITH = 1,
        DAC_MODE_STREAM = 2
    };


    Ui::MainWindow *ui;
    struct ChUi {
        QCheckBox *enBox;
        QComboBox *typeBox;
        QComboBox *rangeBox;
        QComboBox *ampType;
        QDoubleSpinBox *ampBox;
        QDoubleSpinBox *offsBox;
        QDoubleSpinBox *freqBox;
        QDoubleSpinBox *phaBox;
        QDoubleSpinBox *dutyRatio;
    };

    ChUi createChannelUi(QGridLayout *lout, QString name, int row,
                         bool allCh=false);

    LTRModuleSelectionUI *m_devsel;
    LQMeas::LTR35Config *m_curCfg;

    QList<ChUi> m_chUi;

    ChUi m_allChUi;

    AfcFltConfig m_afcCfg;

    LQxtTableLoggerEngine *m_tblLog;
};

#endif // MAINWINDOW_H
