#ifndef LQMEAS_DEVADCCONFIG_H
#define LQMEAS_DEVADCCONFIG_H

#include "DevAdcInfo.h"
#include <QVector>
#include "lqmeas/devs/DeviceConfigIface.h"

namespace LQMeas {
    class DeviceTypeInfo;
    class Device;
    class Unit;

    class DevAdcConfig : public DeviceConfigIface {
        Q_OBJECT
    public:
        DevAdcConfig(const DevAdcInfo &info);
        DevAdcConfig(const DevAdcConfig &cfg);
        virtual ~DevAdcConfig() override;

        const DevAdcInfo *adcInfo() const {return m_adc_info;}

        virtual double adcFreq() const = 0;
        virtual double adcChFreq() const;
        /* начальное смещение по времени заданного канала АЦП относительно первого канала */
        virtual double adcChTimeShift(int ch) const;

        virtual bool adcChEnabled(int ch) const = 0;
        virtual bool adcExternalStart() const {return false;}

        virtual int  adcEnabledChCnt() const {return m_ch_indexes.size();}

        virtual int adcChRangeNum(int ch) const = 0;
        virtual double adcChRangeMaxVal(int ch) const {
            return adcInfo()->adcChRangeMaxVal(ch, adcChRangeNum(ch), adcUnitMode(ch));
        }
        virtual double adcChRangeMinVal(int ch) const {
            return adcInfo()->adcChRangeMinVal(ch, adcChRangeNum(ch), adcUnitMode(ch));
        }

        /* доступен ли канал для разрешения для этой конфигурации и этого типа устройства */
        virtual bool adcChAvailable(int ch_num) const {
            return adcInfo()->adcChAvailable(ch_num);
        }

        virtual int adcUnitMode(int ch) const { Q_UNUSED(ch) return 0;}
        virtual const Unit &adcChRangeUnit(int ch) const {
            return m_adc_info->adcChRangeUnit(ch, adcUnitMode(ch));
        }

        virtual const Unit &adcChValUnit(int ch) const {
            return m_adc_info->adcChValUnit(ch, adcUnitMode(ch));
        }


        /* возможно ли измерение переменной составляющей сигнала по данному каналу */
        virtual bool adcChSupportMeasAC(int ch) const {
            Q_UNUSED(ch)
            return  true;
        }
        /* возможно ли измерение постоянной составляющей сигнала по данному каналу */
        virtual bool adcChSupportMeasDC(int ch) const {
            Q_UNUSED(ch)
            return  true;
        }


        /* получить номер физ. канала по индексу при приеме */
        virtual int adcChNum(int idx) const;
        virtual int adcChIdx(int ch) const;

        double adcDefaultFreq() const;
    Q_SIGNALS:
        void adcChRangeChanged(int ch, double minVal, double maxVal);
        void adcChEnableStateChanged(int ch, bool en);
        void adcChModeChanged(int ch);        
        void adcChConfiguredFreqChanged(int ch);
        void adcChConfigChanged(int ch);
        void adcConfiguredFreqChanged();
    protected:
        QString cfgIfaceName() const override {return  QStringLiteral("Adc");}

        void protUpdate() override;
        void protUpdateInfo(const DeviceTypeInfo *info) override;

        void notifyChEnableStateChanged(int ch);
        void notifyChRangeChanged(int ch);
        void notifyChModeChanged(int ch);
        void notifyChFreqChanged(int ch = -1);
        void notifyAdcFreqChanged();
        void notifyChConfigChanged(int ch);
    private:
        const DevAdcInfo *m_adc_info;
        QVector<int> m_ch_indexes;
        friend class DeviceConfig;
    };
}

#endif // LQMEAS_DEVADCCONFIG_H
