#include "DevAdcStdSeq32Config.h"
#include "lqmeas/EnumConfigKey.h"
#include <QJsonObject>
#include <QJsonArray>
#include "lqmeas/StdConfigKeys.h"

namespace LQMeas {
    static const QLatin1String cfgkey_ch_range_num        {StdConfigKeys::rangeNum()};
    static const QLatin1String cfgkey_ch_mode             {StdConfigKeys::mode()};

    typedef EnumConfigKey<DevAdcStdSeq32Config::AdcChMode> AdcChModeKey;
    static const AdcChModeKey adc_ch_modes[] = {
        {QStringLiteral("comm"),    DevAdcStdSeq32Config::AdcChMode::Comm},
        {QStringLiteral("diff"),    DevAdcStdSeq32Config::AdcChMode::Diff},
        {QStringLiteral("zero"),    DevAdcStdSeq32Config::AdcChMode::Zero},
        {AdcChModeKey::defaultKey(),DevAdcStdSeq32Config::AdcChMode::Comm},
    };


    DevAdcStdSeq32Config::DevAdcStdSeq32Config(const DevAdcInfo &info) : DevAdcSeqBaseConfig{info} {
        memset(m_ch_params, 0, sizeof(m_ch_params));
    }

    DevAdcStdSeq32Config::DevAdcStdSeq32Config(const DevAdcStdSeq32Config &cfg) : DevAdcSeqBaseConfig{cfg} {
        memcpy(m_ch_params, cfg.m_ch_params, sizeof(m_ch_params));
    }

    void DevAdcStdSeq32Config::adcSetChMode(int ch, DevAdcStdSeq32Config::AdcChMode mode) {
        if (m_ch_params[ch].Mode != mode) {
            m_ch_params[ch].Mode = mode;
            notifyConfigChanged();
            Q_EMIT adcChModeChanged(ch);
        }
    }

    void DevAdcStdSeq32Config::adcSetChRangeNum(int ch, int range_num) {
        if (m_ch_params[ch].Range != range_num) {
            m_ch_params[ch].Range = range_num;
            notifyConfigChanged();
            notifyChRangeChanged(ch);
        }
    }

    void DevAdcStdSeq32Config::adcChSave(int ch, QJsonObject &chObj) const     {
        chObj[cfgkey_ch_range_num] = m_ch_params[ch].Range;
        chObj[cfgkey_ch_mode] =AdcChModeKey::getKey(adc_ch_modes, m_ch_params[ch].Mode);
    }

    void DevAdcStdSeq32Config::adcChLoad(int ch, const QJsonObject &chObj)     {
        int range = chObj[cfgkey_ch_range_num].toInt();
        if ((range >= 0) || (range < adcInfo()->adcChRangesCnt(ch, adcUnitMode(ch)))) {
            m_ch_params[ch].Range = range;
        } else {
            m_ch_params[ch].Range = 0;
        }
        m_ch_params[ch].Mode = AdcChModeKey::getValue(
                    adc_ch_modes, chObj[cfgkey_ch_mode].toString());
    }

    void DevAdcStdSeq32Config::adcChSetDefault(int ch)     {
        m_ch_params[ch].Mode = AdcChMode::Comm;
        m_ch_params[ch].Range = 0;
    }

    bool DevAdcStdSeq32Config::adcHasZeroChannels() const {
        bool ret = false;
        for (int ch = 0; (ch < adc_channels_cnt) && !ret; ++ch) {
            if (adcChEnabled(ch) && (adcChMode(ch) == AdcChMode::Zero)) {
                ret = true;
            }
        }
        return ret;
    }
}
